<?php

require_once('cmis_repository_wrapper.php');
global $CFG;
require_once($CFG->libdir.'/filelib.php');

function makecookiefromcasticket(){

	global $CFG;
	//get configuration for CAS
	$casconfig = get_config('auth/cas');
	if($casconfig == NULL || $casconfig->proxycas != 1){
		error(get_string('notcasproxy','referencement'));
	}
	//construct url to connect
	$host = $CFG->block_referencement_host;
	$port = $CFG->block_referencement_port;
	$siteuri = $CFG->block_referencement_site;
	$accesspoint = $CFG->block_referencement_accesspoint;
	$baseURL = 'http://'.str_replace('//','/',$host.":".$port.'/'.$siteuri.'/'.$accesspoint);
	//create CAS proxy
	$CAS = new CASClient($casconfig->version,TRUE,$casconfig->hostname,(Integer)$casconfig->port,$casconfig->baseuri);
	$CAS->setNoCasServerValidation();
	$CAS->forceAuthentication();
	//get a ticket
	if($CAS->hasPGT()){
		$err_code = 0;
		$output = '';
		$casticket = $CAS->retrievePT($baseURL,$err_code,$output);
		if($err_code==3)
		{
			error(get_string('pgtexpired','referencement'));
		}
	}
	else
	{
		error(get_string('notcasproxy','referencement'));
	}
	if(empty($casticket)){
		error("unable to get CAS ticket");
	}


	//add ticket to the url
	$request = $baseURL.'?ticket='.$casticket;
	//add service to the url
	$request .= '&service='.$baseURL;


	$ch = curl_init($request);
	curl_setopt($ch, CURLOPT_RETURNTRANSFER,1);
	curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
	curl_setopt($ch, CURLOPT_HTTPGET, true);
	curl_setopt($ch, CURLOPT_HEADER, true);

	$result = curl_exec($ch);
	$code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
	curl_close($ch);
	if($code != 200){
		return NULL;
	}

	$matches = array();
	preg_match("#Set-Cookie:(.+);#i",$result,$matches);
	if(empty($matches[1]))
	{
		return NULL;
	}
	$cookie = $matches[1];

	return $cookie;
}


class CMISRepositoryClient {
	protected $host,$port,$siteuri,$accesspoint;
	protected $baseURL;
	protected $client;
	public $authenticated;

	function CMISRepositoryClient($login, $password,$host,$port, $siteuri,$accesspoint)
	{
		if($port == 0){
			$port = 80;
		}
		$this->login = $login;
		$this->password = $password;

		$this->host = $host;
		$this->port = $port;
		$this->siteuri = $siteuri;
		$this->accesspoint = $accesspoint;

		//construct URL
		$this->baseURL = 'http://'.str_replace('//','/',$this->host.":".$this->port.'/'.$this->siteuri.'/'.$this->accesspoint);
		try{
			$this->client =& new CMISService($this->baseURL,$login,$password,NULL,'CMISREPOSITORY'.md5($login),'makecookiefromcasticket');
		}catch(Exception $e){
			error(get_string('cmisclienterror','referencement'));
		}
		$this->authenticated = $this->client->authenticated;
	}

	//return file id
	function uploadfile($relativeremotepath,$absolutelocalpath,$filename,$override=false,$newversion=false)
	{
		global $CFG;
		//for conveniance, add a slash at the end of the remote path
		if($relativeremotepath[strlen($relativeremotepath)-1] != '/')
		{
			$relativeremotepath.='/';
		}
		$lowerrelativeremotepath = strtolower($relativeremotepath);

		//check if a same-named file already exists
		$file = $this->client->getObjectByPath($relativeremotepath.$filename);
		if($file === 0){
			error('bad login or server unavaible');
		}
		if(empty($file->id)){
			//test with lowercase remotepath
			$file = $this->client->getObjectByPath($lowerrelativeremotepath.$filename);
		}

		if(!empty($file->id)){
			if($override==false)
			{
				//ask user for wath to do.
				$this->existingfile($filename);
			}
			else
			{
				//delete the existing file,and continue
				$this->client->deleteObject($file->id);
			}
		}

		//check if the remote directory exists, if not: try to create it
		$folder = $this->client->getObjectByPath($relativeremotepath);
		$folderId = $folder->id;
		if(empty($folder->id))
		{
			//test with lowcase
			$folder = $this->client->getObjectByPath($lowerrelativeremotepath);
			$folderId = $folder->id;
			if(empty($folder->id)){
				$folderId  = $this->createfolder($relativeremotepath);
			}
		}

		if(empty($folderId)){
			error(get_string('remotefoldererror','referencement'));
		}

		//get the mime type of the file
		$mimetype = mimeinfo('type', $filename);

		//get the file's contents
		$ffile = fopen($absolutelocalpath,'r');
		$content = stream_get_contents($ffile);
		fclose($ffile);
		if($content == false)
		{
			error(get_string('cannotreadfile','error',$absolutelocalpath));
		}

		//upload
		$file = $this->client->createDocument($folderId,$filename,array(),$content,$mimetype);
		if($file == false){
			error(get_string('cannotcreatedocument','referencement',$relativeremotepath));
		}

		if($file == false)
		{
			error(get_string('cannotuploadfile','error',$absolutelocalpath));
		}


		return $file->id;
	}

	function createfolder($relativeremotepath)
	{

		$root = $this->client->getObjectByPath('/');
		$folds = explode('/',$relativeremotepath);
		$lastid = $root->id;
		$lastfold = '';
		foreach($folds as $f)
		{
			if(empty($f)) continue;

			$lastfold .= '/'.$f;
			$tmp = $this->client->getObjectByPath($lastfold);
			if(empty($tmp->id))
			{
				$tmp = $this->client->createFolder($lastid,$f);
			}
			$lastid = $tmp->id;
			if(empty($lastid)){
				return null;
			}
		}
		return $lastid;
	}

	function existingfile($filename)
	{
		global $CFG;
		$fileext= substr($filename,strrpos($filename,'.'));
		$filename = substr($filename,0,strrpos($filename,'.'));
		$mform = new MoodleQuickForm('existingfile','POST','');
		$mform->addElement('header', 'existingfilehead', get_string('advertise', 'referencement'));
		$mform->addElement('radio','groupaction','',get_string('replace','referencement'),'replace');
		$newnamegroup = array();
		$newnamegroup[] = $mform->createElement('radio','groupaction',null,get_string('createNew','referencement'),'createNew');
		$newnamegroup[] = $mform->createElement('text','newname','','value="'.$filename.'"');
		$newnamegroup[] = $mform->createElement('static','fileext','',$fileext);
		$mform->addGroup($newnamegroup);
		$mform->addElement('radio','groupaction','',get_string('nothing','referencement'),'nothing');
		$mform->addElement('submit', 'validatebutton',get_string('validate','referencement'));
		$mform->addElement('hidden','fileext',$fileext);
		$mform->setDefault('groupaction', 'nothing');
		foreach($_POST as $k=>$v)
		{
			if($k == 'newname' || $k == 'groupaction' || $k == 'validatebutton') continue;
			if(is_array($v))
			{
				foreach($v as $key=>$val){
					$mform->addElement('hidden',$k.'['.$key.']',$val);
				}
			}else{
				$mform->addElement('hidden',$k,$v);
			}
		}
		print_header();
		$mform->display();
		print_footer();die;
	}


}



?>